<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2008 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

function plugin_pollprio_install () {
	api_plugin_register_hook('pollprio', 'config_form', 'pollprio_config_form', 'setup.php');
	api_plugin_register_hook('pollprio', 'api_device_save', 'pollprio_api_device_save', 'setup.php');
	api_plugin_register_hook('pollprio', 'config_settings', 'pollprio_config_settings', 'setup.php');
	api_plugin_register_hook('pollprio', 'device_action_array', 'pollprio_device_action_array', 'setup.php');
	api_plugin_register_hook('pollprio', 'device_action_execute', 'pollprio_device_action_execute', 'setup.php');
	api_plugin_register_hook('pollprio', 'device_action_prepare', 'pollprio_device_action_prepare', 'setup.php');
	api_plugin_register_hook('pollprio', 'poller_command_args', 'pollprio_poller_command_args', 'setup.php');

	pollprio_setup_table ();
}

function plugin_pollprio_uninstall () {
	// Do any extra Uninstall stuff here
}

function plugin_pollprio_check_config () {
	global $config;
	// Here we will check to ensure everything is configured
	pollprio_check_upgrade ();

	include_once($config['library_path'] . '/database.php');
	$r = read_config_option('pollprio_enabled');
	$result = db_fetch_assoc("SELECT * FROM settings WHERE name='pollprio_enabled'");
	if (!isset($result[0]['name']))
		$r = NULL;
	if ($r == '' or !($r =='on' or $r == 'off')) {
		if ($r === '')
			$sql = "REPLACE INTO settings VALUES ('pollprio_enabled','off')";
		else if ($r === NULL)
			$sql = "INSERT INTO settings VALUES ('pollprio_enabled','off')";
		else
			$sql = "UPDATE settings SET value = 'off' WHERE name = 'pollprio_enabled'";
		$result = db_execute($sql) or die (mysql_error());
		kill_session_var('sess_config_array');
	}

	return true;
}

function plugin_pollprio_upgrade () {
	// Here we will upgrade to the newest version
	pollprio_check_upgrade ();
	return false;
}

function pollprio_version () {
	return plugin_pollprio_version();
}

function pollprio_check_upgrade () {
	$current = plugin_pollprio_version ();
	$current = $current['version'];
	$old = read_config_option('plugin_pollprio_version');
	if ($current != $old)
		pollprio_setup_table ();
	// Set the new version
	db_execute("REPLACE INTO settings (name, value) VALUES ('plugin_pollprio_version', '$current')");
}

function plugin_pollprio_version () {
	return array(
		'name'     => 'pollprio',
		'version'  => '0.1',
		'longname' => 'Polling Priority',
		'author'   => 'Jonathan Thibault',
		'homepage' => 'http://cactiusers.org',
		'email'    => 'jonathan@navigue.com',
		'url'      => 'http://versions.cactiusers.org/'
	);
}

function pollprio_config_settings () {
	global $tabs, $settings;

	if (isset($_SERVER['PHP_SELF']) && basename($_SERVER['PHP_SELF']) != 'settings.php')
		return;

	$tabs['poller'] = 'Poller';

	$temp = array();
	foreach ($settings['poller'] as $f => $a) {
		$temp[$f] = $a;
		if ($f == 'max_get_size') {
			$temp['pollprio_enabled'] = array(
				'friendly_name' => 'Enable polling priority',
				'description' => 'This causes the poller to poll hosts in order of priority',
				'method' => 'checkbox',
			);
		}
	}

	$settings['poller']=$temp;
}

function pollprio_poller_command_args ($extra_args) {
	global $config;


	if (read_config_option('pollprio_enabled') == 'on' && read_config_option('poller_type') == '2')
		 $extra_args .= " --pollprio";
	return $extra_args;
}

function pollprio_config_form () {
	global $fields_host_edit;
	$temp = array();
	foreach ($fields_host_edit as $f => $a) {
		$temp[$f] = $a;
		if ($f == 'disabled') {
			$temp['pollprio'] = array(
				'friendly_name' => 'Polling Priority',
				'description' => 'Higher priority hosts will be polled first.',
				'method' => 'drop_array',
				'value' => '|arg1:pollprio|',
				"array" => array(
					"1" => "1 (highest)",
					"2" => "2",
					"3" => "3",
					"4" => "4",
					"5" => "5 (default)",
					"6" => "6",
					"7" => "7",
					"8" => "8",
					"9" => "9 (lowest)"
					)
			);
		}
	}
	$fields_host_edit = $temp;
}

function pollprio_api_device_save ($save) {
	if (isset($_POST['pollprio']))
		$save['pollprio'] = form_input_validate($_POST['pollprio'], 'pollprio', '[1-9]', false, 3);
	return $save;
}

function pollprio_device_action_array ($device_action_array) {
	$device_action_array['pollprio_change'] = 'Change Polling Priority';
	return $device_action_array;
}

function pollprio_device_action_prepare ($save) {
	global $colors;
	$action = $save['drp_action'];
	if ($action != 'pollprio_change')
		return $save;
        // do prepare stuff
	print "<tr><td colspan='2' class='textArea' bgcolor='#" . $colors["form_alternate1"] . "'><p>To " .
		"<select id='pollprio' name='pollprio'><option value='1'>1 (highest)</option>
		<option value='2'>2</option>
		<option value='3'>3</option>
		<option value='4'>4</option>
		<option value='5' selected>5 (default)</option>
		<option value='6'>6</option>
		<option value='7'>7</option>
		<option value='8'>8</option>
		<option value='9'>9 (lowest)</option>
		</select>" .
		" for: </p><p>" . $save['host_list'] . "</p></td></tr>";
}

function pollprio_device_action_execute ($action) {
	if ($action != 'pollprio_change')
		return $action;

	input_validate_input_regex($_POST['pollprio'], '[1-9]');
	$prio = $_POST['pollprio'];
	$selected_items = unserialize(stripslashes($_POST["selected_items"]));

	for ($i = 0; ($i < count($selected_items)); $i++) {
		input_validate_input_number($selected_items[$i]);
		db_execute("update host set pollprio='" . $prio . "' where id='" . $selected_items[$i] . "'");
	}
	return;
}

function pollprio_setup_table () {
	api_plugin_db_add_column ('pollprio', 'host', array('name' => 'pollprio', 'type' => 'tinyint', 'NULL' => false, 'default' => '5', 'after' => 'disabled'));
}

